@@ -74,16 +74,24 @@
 {                                                                                           \
     response = FileOperator::NONE;                                                          \
                                                                                             \
-    if (newFile.exists()) {                                                                 \
+    while (response == FileOperator::NONE && newFile.exists()) {                            \
         if (overwriteAll != FileOperator::NONE) {                                           \
             response = overwriteAll;                                                        \
         } else {                                                                            \
-            bool dirOverDir = false;                                                        \
-            if (newFile.isDir() && file.isDir()) dirOverDir = true;                         \
-            emit showOverwritePrompt(this, newFile.absoluteFilePath(), dirOverDir);         \
+            emit showOverwritePrompt(this, newFile.absoluteFilePath(),                      \
+                newFile.isDir() && file.isDir());                                           \
             waitCond.wait(&mutex);                                                          \
+                                                                                            \
+            if (response == FileOperator::NONE) {                                           \
+                emit showInputFilenamePrompt(this, newFile, file.isDir());                  \
+                waitCond.wait(&mutex);                                                      \
+                if (newNameFromDialog.size()) {                                             \
+                    newFile.setFile(newNameFromDialog);                                     \
+                }                                                                           \
+            }                                                                               \
         }                                                                                   \
     }                                                                                       \
+    if (response == FileOperator::ASK) response = FileOperator::NONE;                       \
 }
 
 
@@ -92,14 +100,33 @@ FileOperator::FileOperator(QWidget *parent) : QWidget(parent) {
     layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
     setLayout(layout);
+    qRegisterMetaType<QFileInfo>("QFileInfo");
 }
 
 
 QString FileOperator::shortenPath(const QString &path) {
     QString homePath = QFSFileEngine::homePath();
-    QString result = path;
+
     if (path.indexOf(homePath, 0) == 0) {
+        QString result = path;
+
         result.replace(0, homePath.size(), "~");
+        return result;
+    }
+
+    return path;
+}
+
+
+QString FileOperator::unwindPath(const QString &path) {
+    QString result = path;
+    // if ~ is the first character and / or nothing follows it, replace with home dir
+    if (path == "~" || path.indexOf("~/", 0) == 0) {
+        QString homePath = QFSFileEngine::homePath();
+        result.replace(0, 1, homePath);
+    // in case someone wants to enter a dir called ~ in the current dir, he can escape it with \~
+    } else if (path == "\\~" || path.indexOf("\\~/", 0) == 0) {
+        result.replace(0, 2, "~");
     }
 
     return result;
@@ -227,6 +254,7 @@ void FileOperator::showOverwritePrompt(
     QAbstractButton *noButton = msgBox.addButtonSecond(QDialogButtonBox::No);
     QAbstractButton *noToAllButton = msgBox.addButtonSecond(QDialogButtonBox::NoToAll);
     QAbstractButton *abortButton = msgBox.addButtonSecond(tr("Abort"), QDialogButtonBox::DestructiveRole);
+    QAbstractButton *newNameButton = msgBox.addButtonFirst(tr("New Name"), QDialogButtonBox::AcceptRole);
     QAbstractButton *askButton = 0;
     QAbstractButton *skipDirButton = 0;
 
@@ -252,13 +280,55 @@ void FileOperator::showOverwritePrompt(
     } else if (msgBox.clickedButton == noToAllButton) {
         manipulator->setResponse(KEEP, true);
     } else if (msgBox.clickedButton == askButton) {
-        manipulator->setResponse(NONE, true);
+        manipulator->setResponse(ASK);
+    } else if (msgBox.clickedButton == newNameButton) {
+        manipulator->setResponse(NONE);
     } else if (msgBox.clickedButton == skipDirButton) {
         manipulator->setResponse(SKIP_DIR);
     }
 }
 
 
+void FileOperator::showInputFilenamePrompt(FileManipulatorThread* manipulator,
+    const QFileInfo &file,
+    const bool dir)
+{
+    bool ok;
+    QString prompt, error;
+
+    if (dir) {
+        prompt = tr("Enter the new directory name.");
+    } else {
+        prompt = tr("Enter the new file name.");
+    }
+
+    manipulator->mutex.lock();
+
+    manipulator->newNameFromDialog = "";
+    QString text = file.fileName();
+
+    
+    while (true) {
+        text = QInputDialog::getText(this, QString(), prompt + error, QLineEdit::Normal, text, &ok);
+
+        if (!ok) break;
+
+        error = "";
+        if (text.contains(QRegExp("[\"*/:<>?\\\\|]"))) {
+            error = "<small><br/><font color = 'red'>" + tr("The name cannot contain any of the following characters: ") +
+                "\"*/:&lt;&gt;?\\|</font></small>";
+        } else if (ok && !text.isEmpty()) {
+            QFileInfo info(file.path() + "/" + text);
+            manipulator->newNameFromDialog = info.absoluteFilePath();
+            break;
+        }
+    }
+
+    manipulator->mutex.unlock();
+    manipulator->waitCond.wakeAll();
+}
+
+
 void FileOperator::remove(FileManipulatorThread* manipulator) {
     manipulator->wait();
     layout()->removeWidget(manipulator->progressBar);
@@ -288,6 +358,8 @@ void FileOperator::caterNewThread(FileManipulatorThread *thread) {
         this, SLOT(showErrorPrompt(FileManipulatorThread*, const QString&, const QString&, const int)));
     connect(thread, SIGNAL(showOverwritePrompt(FileManipulatorThread*, const QString&, bool)),
         this, SLOT(showOverwritePrompt(FileManipulatorThread*, const QString&, bool)));
+    connect(thread, SIGNAL(showInputFilenamePrompt(FileManipulatorThread*, const QFileInfo&, bool)),
+        this, SLOT(showInputFilenamePrompt(FileManipulatorThread*, const QFileInfo&, bool)));
     connect(thread, SIGNAL(finished(FileManipulatorThread*)),
         this, SLOT(remove(FileManipulatorThread*)));
     connect(thread, SIGNAL(setBarSize(FileManipulatorThread*, unsigned int)),
@@ -295,8 +367,6 @@ void FileOperator::caterNewThread(FileManipulatorThread *thread) {
     connect(thread, SIGNAL(updateProgress(FileManipulatorThread*, int)),
         this, SLOT(updateProgress(FileManipulatorThread*, int)));
 
-    thread->progressBar->setValue(0);
-
     layout()->addWidget(thread->progressBar);
     thread->start(QThread::LowestPriority);
 }
@@ -317,7 +387,9 @@ FileManipulatorThread::FileManipulatorThread(const QFileInfoList files, QDir des
     fileValue(0)
 {
     memset(ignoreAll, false, sizeof(ignoreAll));
-    progressBar->setMaximum(0);
+    progressBar->setMaximum(1);
+    progressBar->setValue(0);
+    progressBar->setMinimum(0);
     QFont barFont = progressBar->font();
     barFont.setPointSize(12);
     progressBar->setFont(barFont);
@@ -428,10 +500,8 @@ void FileManipulatorThread::copy(const QFileInfo &file) {
         << " to " << dest.absolutePath().toStdString() << std::endl;
 
     QString path(file.absoluteFilePath());
-    QString newPath(dest.absolutePath() + "/" + file.fileName());
     QFSFileEngine engine(path);
-    QFSFileEngine newEngine(newPath);
-    QFileInfo newFile(newPath);
+    QFileInfo newFile(dest.absolutePath() + "/" + file.fileName());
 
     updateFile(path);
 
@@ -442,6 +512,9 @@ void FileManipulatorThread::copy(const QFileInfo &file) {
         OVERWRITE_PROMPT(file, newFile)
     }
 
+    QString newPath(newFile.absoluteFilePath());
+    QFSFileEngine newEngine(newPath);
+
     if (abort) return;
 
     if (file.isDir()) {
@@ -732,13 +805,15 @@ void MoveThread::rename(const QFileInfoList &files, const QDir &dest) {
     for (int i = 0; i < files.size(); ++i) {
         QString path = files[i].absoluteFilePath();
         QFSFileEngine engine(path);
-        QString newPath = dest.absolutePath() + "/" + files[i].fileName();
-        QFileInfo newFile(newPath);
+        QFileInfo newFile(dest.absolutePath() + "/" + files[i].fileName());
 
         updateFile(path);
 
         OVERWRITE_PROMPT(files[i], newFile)
 
+        // if we are owerwriting dir over a dir, we will get SKIP_DIR
+        // as a response from OVERWRITE_PROMT meaning we should skip it
+        // (KEEP would mean to keep the files inside)
         if (files[i].isDir() && newFile.exists() && newFile.isDir()) {
             if (response == FileOperator::SKIP_DIR) {
                 if (abort) break;
@@ -755,6 +830,9 @@ void MoveThread::rename(const QFileInfoList &files, const QDir &dest) {
             }
         }
 
+        QString newPath(newFile.absoluteFilePath());
+        QFSFileEngine newEngine(newPath);
+
         while (!abort && !engine.rename(newPath)) {
             // source and target are on different partitions
             // this should happen on the first file, unless some are skipped by overwrite prompt
