@@ -37,11 +37,12 @@ class FileOperator : public QWidget {
 
 public:
     // DONT_ASK_ONCE is a hackish way to avoid asking twice to overwrite the same directory when moving
-    enum Response{NONE, ABORT, RETRY, IGNORE, KEEP, OVERWRITE, SKIP_DIR, DONT_ASK_ONCE};
+    enum Response{NONE, ABORT, RETRY, IGNORE, KEEP, OVERWRITE, SKIP_DIR, ASK, DONT_ASK_ONCE};
 
     FileOperator(QWidget *parent = 0);
 
     static QString shortenPath(const QString &path);
+    static QString unwindPath(const QString &path);
 
     void deleteFiles(const QFileInfoList &files);
     void copyFiles(const QFileInfoList &files, QDir &destination);
@@ -55,6 +56,9 @@ public slots:
     void showOverwritePrompt(FileManipulatorThread* manipulator,
         const QString &fileName,
         const bool dirOverDir);
+    void showInputFilenamePrompt(FileManipulatorThread* manipulator,
+        const QFileInfo &fileName,
+        const bool dirOverDir);
 
     void remove(FileManipulatorThread* manipulator);
     void setBarSize(FileManipulatorThread* manipulator, unsigned int size);
@@ -80,6 +84,10 @@ public:
     QProgressBar *progressBar;
 
     time_t startTime;
+    QMutex mutex;
+    QWaitCondition waitCond;
+    // the new name entered from the overwrite dialog
+    QString newNameFromDialog;
 
 protected:
     void processFiles(const QFileInfoList &files);
@@ -101,31 +109,40 @@ protected:
     void updateProgress(int value);
     void updateFile(const QString &name);
 
+    // files to process by the operation
     const QFileInfoList files;
+    // destination for files - changes as the operation recurses into directories
     QDir dest;
 
+    // responses from the dialog prompts (error and overwrite)
     FileOperator::Response response;
     FileOperator::Response overwriteAll;
+    // a flag to abort the operation
     bool abort;
+    // an array indicating whether to always ignore the error of index errno
     bool ignoreAll[256];
 
     // set of files that won't be deleted by the remove(...) functions
     // used when move(...) would not overwrite target file to ensure the source file doesn't get deleted
     QSet<QString> removeExcludeFiles;
 
+    // A map of file paths to their size. Not the actual size, but what is calculated for the
+    // purpose of the progressbar for the given operation. So either fileSize/BLOCK_SIZE or simply
+    // 1 for a file and file count for dirs (or both for copy&delete)
     QMap<QString, qint64> fileSizeMap;
 
-    QMutex mutex;
-    QWaitCondition waitCond;
-
+    // the name of the file thats being processed (for progressBar) and the text of the progressBar (the format)
     QString fileName, barText;
+    // stamp of the last ETA recalculation - done every second
     time_t lastTimeUpdate;
     char timeBuf[10];
+    // progress information of the bar and for the current file
     unsigned int barSize, barValue, fileSize, fileValue;
 
 signals:
     void showErrorPrompt(FileManipulatorThread*, const QString&, const QString&, const int);
     void showOverwritePrompt(FileManipulatorThread*, const QString&, const bool);
+    void showInputFilenamePrompt(FileManipulatorThread*, const QFileInfo&, const bool);
     void finished(FileManipulatorThread*);
     void setBarSize(FileManipulatorThread*, unsigned int);
     void updateProgress(FileManipulatorThread*, int);
